import json
import re

from .VKBotLongPoll import EventInformation

commands = {}
json_commands = {}
search_commands = {}


def command_handler(*args):
    def decorator(fn):
        for i in args:
            commands[i.lower()] = fn

    return decorator


def search_command_handler(*args):
    def decorator(fn):
        for i in args:
            search_commands[i[0].lower()] = [fn, i[1]]

    return decorator


def json_command_handler(*args):
    def decorator(fn):
        for i in args:
            commands[i.lower()] = fn
            json_commands[i] = fn

    return decorator


class CommandHandler:
    def check_updates(self, raw: EventInformation):
        lower = raw.text.lower()
        if commands.get(lower) is not None:
            commands[lower](raw)
            return

        for i in search_commands:
            if re.search(i, lower):
                splited = raw.text.split(search_commands[i][1])
                if len(splited) > 1:
                    search_commands[i][0](raw, splited_text=splited)
                    return

        if raw.payload is not None:
            payload_data = raw.payload
            load = json.loads(payload_data)
            button = load.get("button")
            if json_commands.get(button) is not None:
                json_commands[button](raw)
            return



import datetime
import os
import multiprocessing

from vk_dark_library.VKUtils import VKcolors, DEBBUGER_VERSION

first_start = []
texts = {
    "start_dev": f"\nDebbuger version {DEBBUGER_VERSION}\n{VKcolors.FAIL}Используйте этот режим только для тестирования!",
    "restart_dev": f'\nБыли обнаружены изменения\nDebbuger version {DEBBUGER_VERSION}\nКод был перезагружен'
}
times = {}


class InitializeDebugger:
    def __init__(self, main_function):
        """

        :param main_function: your function that will be called
        """
        self.__main_function = main_function
        self.__total_size = 0
        self.__old_size = 0
        self.__main_thread = multiprocessing.Process(target=main_function)

    def __run(self, isfirst=False):
        if isfirst:
            date = datetime.datetime.now().strftime('%b %d, %Y %H:%M:%S')
            print(f"{VKcolors.HEAD}[{date}] - {texts['start_dev']}{VKcolors.END}")
        self.__main_thread.start()

    def __restart(self):
        date = datetime.datetime.now().strftime('%b %d, %Y %H:%M:%S')
        print(f"{VKcolors.HEAD}[{date}] - {texts['restart_dev']}{VKcolors.END}")
        self.__main_thread.terminate()
        self.__main_thread = multiprocessing.Process(target=self.__main_function, name="main_function")
        self.__run()

    def start_debug(self):
        while True:
            restructed = False
            for path, dirs, files in os.walk(os.getcwd()):
                for file in files:
                    f_j = os.path.join(path, file)
                    if f_j.endswith(".py"):
                        if times.get(f_j) != os.path.getmtime(f_j):
                            times[f_j] = os.path.getmtime(f_j)
                            restructed = True
            if not restructed:
                continue
            if not first_start:
                first_start.append(True)
                self.__run(isfirst=True)
                continue

            self.__restart()



class BaseException(Exception):
    def __init__(self, *args):
        if args:
            self.error = args[0]
            return
        self.error = "Critical error!"

    def __str__(self):
        return self.error


class LongpollConnectionError(BaseException):
    pass


class ApiMethodError(BaseException):
    pass



import datetime
import importlib
import os
import sys

import requests
from vk_dark_library.VKErrorLibrary import LongpollConnectionError, ApiMethodError
from vk_dark_library.VKUtils import VKcolors, LONGPOLL_VERSION

File_url = None
vk_api_url = "https://api.vk.com/method/"
v = '5.130'
config = {"v": "5.130", "token": None, "group": None, "debug": None}
NEW = "message_new"


class LoadConfig:
    def __init__(self, path, name):
        if path is not None:
            sys.path.append(path)
            config_file = importlib.import_module(name)
            config.update({"token": config_file.token, "group": config_file.group_id, "debug": config_file.debug})


class EventInformation:
    __slots__ = ["payload", "clear_query", "group_id", "chat_id", "message", "type", "text", "from_id", "peer_id",
                 "conversation_message_id", "send_id", "splited_text", "lower"]

    def __init__(self, raw_query, splited_text=None):
        if config["token"] is None:
            raise Exception("Config not loaded")
        if splited_text is not None:
            self.splited_text = splited_text
        self.clear_query = raw_query["object"]
        self.type = raw_query['type']
        self.group_id = raw_query["group_id"]
        self.chat_id = self.clear_query['message']['peer_id']
        self.from_id = self.clear_query["message"]["from_id"]
        if self.chat_id > 2000000000:
            self.chat_id = self.chat_id - 2000000000
        self.peer_id = self.clear_query['message']['peer_id']
        self.message = self.clear_query['message']
        self.text = self.clear_query['message']['text']
        self.lower = self.text.lower()
        self.payload = self.clear_query['message'].get("payload")
        self.conversation_message_id = self.clear_query["message"]["conversation_message_id"]
        if self.peer_id == self.from_id:
            self.send_id = {"user_id": self.chat_id}
            return
        self.send_id = {"chat_id": self.chat_id}


class LogJournal:
    def insert_new_error_line(self, line):
        with open(os.getcwd() + "\\vk_dark_errors.log", "a", encoding="UTF-8") as file:
            file.write(line)

    def insert_new_log_line(self, line):
        with open(os.getcwd() + "\\vk_dark_access.log", "a", encoding="UTF-8") as file:
            file.write(line)


class VKLongPoll:
    def __init__(self):
        print(f"{VKcolors.HEAD}Longpoll version {LONGPOLL_VERSION}{VKcolors.END}\n")
        self._ts = []
        if config["token"] is None:
            raise Exception("Config not loaded")
        self._lpserver = self.__update_server()
        if self._lpserver.get("error") is not None:
            raise LongpollConnectionError("The group_id or token is specified incorrectly in config.")
        self._url = None
        self._key = self._lpserver["response"]["key"]
        self._server = self._lpserver["response"]["server"]
        self._error = None
        self._session = requests.Session()
        self._ts.append(self._lpserver["response"]["ts"])

    def __update_server(self):
        payload = {
            'v': v,
            'group_id': config["group"],
            'access_token': config["token"]
        }
        return requests.get(url=vk_api_url + "groups.getLongPollServer", params=payload).json()

    def __one_listen(self):
        response = self._lpserver
        response_data = response['response']
        if len(self._ts) == 0:
            self._ts.append(response_data['ts'])
        payload = {
            'act': 'a_check',
            'key': str(self._key),
            'wait': '5',
            'ts': str(self._ts[0])
        }
        update_data = requests.post(url=self._server, data=payload)
        if update_data.json().get("failed") is not None:
            error = update_data.json()["failed"]
            updated_data = self.__update_server()
            if error == 2:
                self._key = updated_data["response"]["key"]
            return update_data
        self._ts = [update_data.json()['ts']]
        return update_data

    def listen(self):
        while True:
            call = self.__one_listen().json()
            if call.get("failed") is None:
                for information in call["updates"]:
                    if not config["debug"]:
                        yield EventInformation(information)
                    else:
                        debug_info = information["object"]["message"]
                        send_text = f"[{debug_info['peer_id']}] {debug_info['from_id']}: "
                        if debug_info["peer_id"] == debug_info["from_id"]:
                            send_text = f"{debug_info['from_id']}: "
                        date = datetime.datetime.now().strftime('%b %d, %Y %H:%M:%S')
                        print(f'[{date}] {VKcolors.HEAD}{send_text}{VKcolors.OKBLUE}{debug_info["text"]}{VKcolors.END}')
                        LogJournal().insert_new_log_line(f"[{date}] {send_text} {debug_info['text']}\n")
                        yield EventInformation(information)


class GetApi:
    __slots__ = ['_method', '__api_token', '__v']

    def __init__(self, method=None):
        if config["token"] is None:
            raise Exception("Config not loaded")
        self.__api_token = config["token"]
        self.__v = "5.130"
        self._method = method

    def __getattr__(self, method):
        if self._method is None:
            return GetApi(method=f"{method}")
        return GetApi(method=f"{self._method}.{method}")

    def __call__(self, **kwargs):
        """
        :param method: Метод выполняемый в вк апи
        :param kwargs: Аргументы для этого метода
        :return: https://vk.com/dev/methods
        """
        success = f"{VKcolors.OKGREEN}успешно"
        if self._method is None:
            self.__return_traceback('Method not entered')
        kwargs.update({"v": self.__v, "access_token": self.__api_token})
        rw = requests.get(vk_api_url + self._method, params=kwargs)
        if rw.json().get('error'):
            text_s = rw.json()["error"]["error_msg"]
            self.__return_traceback(text_s)
            success = f"{VKcolors.FAIL}ошибкой"
        date = datetime.datetime.now().strftime('%b %d, %Y %H:%M:%S')
        LogJournal().insert_new_log_line(f"[{date}] Вызван  метод {self._method}\n")
        print(f"Выполнение запроса '{self._method}' закончилось {success}. {VKcolors.END}\n")
        return rw.json()

    def __return_traceback(self, text_s):
        date = datetime.datetime.now().strftime('%b %d, %Y %H:%M:%S')
        if config["debug"]:
            print(f"[{date}] {VKcolors.FAIL}[ERR] {VKcolors.OKCYAN}{text_s}{VKcolors.END}")
            LogJournal().insert_new_error_line(f"[{date}] {text_s}\n")
            return
        raise ApiMethodError(text_s)



from setuptools import setup, find_packages

with open("README.md", "r", encoding="UTF-8") as readme_file:
    readme = readme_file.read()

setup(name='vk_dark_library',
      version='1.0.6',
      url='https://github.com/DarkLorianPrime/vk_library',
      license='Apache License, Version 2.0, see LICENSE file',
      author='Darklorian (Aleksander Kasimov)',
      python_requires='>3.6',
      author_email='lolpokens@mail.ru',
      description='Быстрое и удобное создание ботов в вк. Пока только LongPoll.',
      packages=find_packages(),
      long_description=readme,
      install_requires=["requests>=2"])
